library sdltest;

{$mode objfpc}{$H+}

uses
  Classes, DGLIntf, DGLGuestIntf, SDL, dglOpenGL
  { you can add units after this };

type

  { TSDLBenchmark }

  TSDLBenchmark = class (TDGLBenchmark)
  protected
    FSurface: PSDL_Surface;
    FTimer: IBenchmarkTimer;
  protected
    procedure DoFinish; override;
    function DoPrepare(AHost: IBenchmarkHost): TBenchmarkResult; override;
    function DoRunBenchmark(ResultMap: IBenchmarkResultMap): TBenchmarkResult;
       override;
    function GetDescription: String; override;
    function GetName: String; override;
  end;

  { TSDLOpenGLBenchmark }

  TSDLOpenGLBenchmark = class (TSDLBenchmark)
  protected
    procedure DoFinish; override;
    function DoPrepare(AHost: IBenchmarkHost): TBenchmarkResult; override;
    function DoRunBenchmark(ResultMap: IBenchmarkResultMap): TBenchmarkResult;
       override;
    function GetDescription: String; override;
    function GetName: String; override;
    procedure RenderFrame;
  end;

  { TLibrary }

  TLibrary = class (TDGLBenchmarkLib)
  protected
    procedure DoHello(AHost: IBenchmarkHost); override;
  end;

{ TSDLBenchmark }

procedure TSDLBenchmark.DoFinish;
begin
  SDL_Quit;
  inherited DoFinish;
end;

function TSDLBenchmark.DoPrepare(AHost: IBenchmarkHost): TBenchmarkResult;
var
  VideoFlags: Cardinal;
  VideoInfo: PSDL_VideoInfo;
begin
  Result:=inherited DoPrepare(AHost);
  FTimer := AHost.GetTimer;
  if FTimer = nil then
    Exit(brInsufficientHostEnvironment);
  SDL_Init(SDL_INIT_VIDEO);
  SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 8);
  SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 8);
  SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 8);
  SDL_GL_SetAttribute(SDL_GL_ALPHA_SIZE, 8);
  SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 16);
  SDL_GL_SetAttribute(SDL_GL_STENCIL_SIZE, 8);

  VideoInfo := SDL_GetVideoInfo;

  VideoFlags := SDL_OPENGL or SDL_HWPALETTE or SDL_DOUBLEBUF;
  if (VideoInfo <> nil) and (VideoInfo^.hw_available <> 0) then
  begin
    VideoFlags := VideoFlags or SDL_HWSURFACE;
  end
  else
  begin
    VideoFlags := VideoFlags or SDL_SWSURFACE;
  end;

  if (VideoInfo <> nil) and (VideoInfo^.blit_hw <> 0) then VideoFlags := SDL_HWACCEL;

  FSurface := SDL_SetVideoMode(800, 600, 32, VideoFlags);
  if FSurface = nil then
    Exit(brInsufficientHardware);

  Result := brOkay;
end;

function TSDLBenchmark.DoRunBenchmark(ResultMap: IBenchmarkResultMap
  ): TBenchmarkResult;
var
  Time: Double;
begin
  FTimer.Reset;
  SDL_Delay(1000);
  Time := FTimer.GetInterval;
  ResultMap.SetResultFloat(ResultMap.GetDefaultResultName(BRK_TIME), Time);
  Result := brOkay;
end;

function TSDLBenchmark.GetDescription: String;
begin
  Result := 'Waits 1s using SDL_Delay.';
end;

function TSDLBenchmark.GetName: String;
begin
  Result := 'SDL_Delay test';
end;

{ TSDLOpenGLBenchmark }

procedure TSDLOpenGLBenchmark.DoFinish;
begin
  inherited DoFinish;
end;

function TSDLOpenGLBenchmark.DoPrepare(AHost: IBenchmarkHost
  ): TBenchmarkResult;
begin
  Result:=inherited DoPrepare(AHost);
  if Result <> brOkay then
    Exit;
  if not InitOpenGL() then
    Exit(brUnknownError);
  ReadOpenGLCore;
  ReadExtensions;

  glViewport(0, 0, 800, 600);
  glMatrixMode(GL_PROJECTION);
    glLoadIdentity;
    //gluPerspective(60.0, 800/600, 1.0, 10.0);
    glOrtho(0, 800, 600, 0, -1.0, 1.0);
  glMatrixMode(GL_MODELVIEW);

  glDisable(GL_CULL_FACE);
end;

function TSDLOpenGLBenchmark.DoRunBenchmark(ResultMap: IBenchmarkResultMap
  ): TBenchmarkResult;
var
  Time: Double;
  Frame: Integer;
  Event: TSDL_Event;
const
  FRAME_COUNT = 1000;
begin
  Time := 0.0;
  for Frame := 1 to FRAME_COUNT do
  begin
    while SDL_PollEvent(@Event) <> 0 do;
    FTimer.Reset;
    RenderFrame;
    Time += FTimer.GetInterval;
  end;
  ResultMap.SetResultFloat(ResultMap.GetDefaultResultName(BRK_TIME), Time);
  ResultMap.SetResultFloat(ResultMap.GetDefaultResultName(BRK_FPS), FRAME_COUNT / Time);
  Result := brOkay;
end;

function TSDLOpenGLBenchmark.GetDescription: String;
begin
  Result := 'A simple OpenGL rendering test.';
end;

function TSDLOpenGLBenchmark.GetName: String;
begin
  Result := 'OpenGL test';
end;

procedure TSDLOpenGLBenchmark.RenderFrame;
var
  Stripe: Integer;
begin
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

  glLoadIdentity;

  glBegin(GL_QUAD_STRIP);
    glColor3f(1, 0, 0);
    glVertex2f(0, 0);
    glColor3f(0, 0, 1);
    glVertex2f(0, 100);
    for Stripe := 1 to 100 do
    begin
      if Odd(Stripe) then
      begin
        glColor3f(0, 1, 0);
        glVertex2f(Stripe * 5, 0);
        glColor3f(1, 1, 1);
        glVertex2f(Stripe * 5, 100);
      end
      else
      begin
        glColor3f(1, 0, 0);
        glVertex2f(Stripe * 5, 0);
        glColor3f(0, 0, 1);
        glVertex2f(Stripe * 5, 100);
      end;
    end;
  glEnd;

  glFlush;

  SDL_GL_SwapBuffers;
end;

{ TLibrary }

procedure TLibrary.DoHello(AHost: IBenchmarkHost);
begin
  AHost.Add(TSDLBenchmark.Create);
  AHost.Add(TSDLOpenGLBenchmark.Create);
end;

procedure Hello(out Intf: IBenchmarkLib);
begin
  Intf := TLibrary.Create;
end;

exports
  Hello;

begin
end.

