library simple;

{$mode objfpc}{$H+}

uses
  SysUtils, Classes, DGLIntf, DGLGuestIntf;

type

  { TSimpleBenchmark }

  TSimpleBenchmark = class (TDGLBenchmark)
  private
    FTimer: IBenchmarkTimer;
  protected
    function BenchmarkNeeds: TBenchmarkNeeds; override;
    procedure DoFinish; override;
    function DoPrepare(AHost: IBenchmarkHost): TBenchmarkResult; override;
    function DoRunBenchmark(ResultMap: IBenchmarkResultMap): TBenchmarkResult;
       override;
    function GetDescription: String; override;
    function GetName: String; override;
  end;

  { TLibrary }

  TLibrary = class (TDGLBenchmarkLib)
  protected
    procedure DoHello(AHost: IBenchmarkHost); override;
  end;

{ TSimpleBenchmark }

function TSimpleBenchmark.BenchmarkNeeds: TBenchmarkNeeds;
begin
  Result:=[];
end;

procedure TSimpleBenchmark.DoFinish;
begin
  FTimer := nil;
end;

function TSimpleBenchmark.DoPrepare(AHost: IBenchmarkHost): TBenchmarkResult;
begin
  FTimer := AHost.GetTimer;
  Result := inherited;
end;

function TSimpleBenchmark.DoRunBenchmark(ResultMap: IBenchmarkResultMap
  ): TBenchmarkResult;
var
  Interval: Double;
begin
  FTimer.Reset;
  Sleep(1000);
  Interval := FTimer.GetInterval;
  ResultMap.SetResultFloat(ResultMap.GetDefaultResultName(BRK_TIME), Interval);
  Result := brOkay;
end;

function TSimpleBenchmark.GetDescription: String;
begin
  Result:='Just a simple benchmark.';
end;

function TSimpleBenchmark.GetName: String;
begin
  Result:='Simple benchmark';
end;

{ TLibrary }

procedure TLibrary.DoHello(AHost: IBenchmarkHost);
begin
  Host.Add(TSimpleBenchmark.Create);
end;

procedure Hello(out Intf: IBenchmarkLib);
begin
  Intf := TLibrary.Create;
end;

exports
  Hello;

{$IFDEF WINDOWS}{$R simple.rc}{$ENDIF}

begin
end.

