unit DGLGuestIntf;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, DGLIntf;

type
  { TDGLBenchmark }

  TDGLBenchmark = class (TInterfacedObject, IBenchmarkGuest, IBenchmarkGuestPas)
  protected
    function BenchmarkNeeds: TBenchmarkNeeds; virtual;
    procedure DoFinish; virtual;
    function DoPrepare(AHost: IBenchmarkHost): TBenchmarkResult; virtual;
    function DoRunBenchmark(ResultMap: IBenchmarkResultMap): TBenchmarkResult; virtual; abstract;
    function GetDescription: String; virtual; abstract;
    function GetName: String; virtual; abstract;
  public
    {$I DGLIntfCalling.inc}
    procedure Finish;
    procedure GetDescription(Buffer: PChar; var Len: LongInt);
    procedure GetName(Buffer: PChar; var Len: LongInt);
    function GetNeeds: TBenchmarkNeeds;
    function Prepare(AHost: IBenchmarkHost): TBenchmarkResult;
    function RunBenchmark(ResultMap: IBenchmarkResultMap): TBenchmarkResult;
    {$CALLING register}
  end;

  { TDGLBenchmarkLib }

  TDGLBenchmarkLib = class (TInterfacedObject, IBenchmarkLib, IBenchmarkHost)
  private
    FHost: IBenchmarkHost;
  protected
    procedure DoHello(AHost: IBenchmarkHost); virtual;
  public
    {$I DGLIntfCalling.inc}
    procedure Hello(Host: IBenchmarkHost);
    {$CALLING register}
  published
    property Host: IBenchmarkHost read FHost implements IBenchmarkHost;
  end;

implementation

{ TDGLBenchmark }

function TDGLBenchmark.BenchmarkNeeds: TBenchmarkNeeds;
begin
  Result := [];
end;

procedure TDGLBenchmark.DoFinish;
begin

end;

function TDGLBenchmark.DoPrepare(AHost: IBenchmarkHost): TBenchmarkResult;
begin
  Result := brOkay;
end;

{$I DGLIntfCalling.inc}

procedure TDGLBenchmark.Finish;
begin
  DoFinish;
end;

procedure TDGLBenchmark.GetDescription(Buffer: PChar; var Len: LongInt);
var
  Tmp: String;
  FinalLen: LongInt;
begin
  if Buffer = nil then
    Len := Length(GetDescription)
  else
  begin
    Tmp := GetDescription;
    FinalLen := Length(Tmp)+1; // Include the #0 char
    if Len < FinalLen then
      FinalLen := Len;
    Move(Buffer^, Tmp[1], FinalLen);
  end;
end;

procedure TDGLBenchmark.GetName(Buffer: PChar; var Len: LongInt);
var
  Tmp: String;
  FinalLen: LongInt;
begin
  if Buffer = nil then
    Len := Length(GetName)
  else
  begin
    Tmp := GetName;
    FinalLen := Length(Tmp)+1; // Include the #0 char
    if Len < FinalLen then
      FinalLen := Len;
    Move(Buffer^, Tmp[1], FinalLen);
  end;
end;

function TDGLBenchmark.GetNeeds: TBenchmarkNeeds;
begin
  Result := BenchmarkNeeds;
end;

function TDGLBenchmark.Prepare(AHost: IBenchmarkHost): TBenchmarkResult;
begin
  Result := DoPrepare(AHost);
end;

function TDGLBenchmark.RunBenchmark(ResultMap: IBenchmarkResultMap
  ): TBenchmarkResult;
begin
  Result := DoRunBenchmark(ResultMap);
end;

{$CALLING register}

{ TDGLBenchmarkLib }
procedure TDGLBenchmarkLib.DoHello(AHost: IBenchmarkHost);
begin

end;

{$I DGLIntfCalling.inc}

procedure TDGLBenchmarkLib.Hello(Host: IBenchmarkHost);
begin
  FHost := Host;
  DoHello(Host);
end;
{$CALLING register}

end.

