unit DGLIntf;

{$mode objfpc}{$H+}
{$I DGLIntfCalling.inc}

interface

uses
  Classes, SysUtils;

type
  TBenchmarkResultKey = type Cardinal;

const
  BRK_NONE = 0;
  BRK_FPS = 1;
  BRK_TIME = 2;
  BRK_MESSAGE = 3;
  BRK_IMAGE = 4;
  BRK_IMAGE_MIME = 5;

  BRKC_GENERIC = 'generic';
  // Stuff in this category won't be shown directly and will be stored base64
  // encoded.
  BRKC_DATA = 'data';
  // This category can be used to set the MIME-type of data in the 'data'
  // category explicitly.
  BRKC_MIME = 'mime';
  // Data in this group will be averaged over multiple runs and a std-dev will
  // be calculated.
  BRKC_PERFORMANCE = 'perf';

  BRKN_FPS = 'perf/fps';
  BRKN_TIME = 'perf/time';
  BRKN_MESSAGE = 'generic/message';
  BRKN_IMAGE = 'data/image';
  BRKN_IMAGE_MIME = 'mime/image';

type
  {$PACKENUM 2}
  {$PACKSET 2}
  TBenchmarkNeed = (bnOpenGL);
  TBenchmarkNeeds = set of TBenchmarkNeed;
  IBenchmarkHost = interface;

  TBenchmarkResult = (brOkay, brUnknownError, brInsufficientHardware,
    brAbortedByUser, brInsufficientHostEnvironment);

  // This is the interface specification for a generic map which should be
  // supplied so that benchmarks can give their results.
  IBenchmarkResultMap = interface ['{A8B056B9-5A38-4294-810A-96EA538B8873}']
    // The PChar returned by this function is constant and must not be freed by
    // the library. So it can be passed to SetResult directly.
    function GetDefaultResultName(const Key: TBenchmarkResultKey): PChar;
    procedure SetResult(const Key: PChar; const Value: PChar; Len: LongInt);
    procedure SetResultInt(const Key: PChar; const Value: LongInt);
    procedure SetResultFloat(const Key: PChar; const Value: Double);
    procedure UnsetResult(const Key: PChar);
  end;

  IBenchmarkParameterMap = interface ['{708E0047-D3EC-4553-A2AA-94B23FB4EB66}']
    // You must NEVER EVER free a value returned by this function.
    function GetValue(const Key: PChar): PChar;
  end;

  // One instance of this interface represents exactly one benchmark.
  // The host will query various stuff about the benchmark using this interface,
  // as it also calls it to initialize the benchmark and so on.
  IBenchmarkGuest = interface ['{D4E46F54-9D3A-4FD3-BBCA-9C821701CB3D}']
    procedure Finish;
    procedure GetDescription(Buffer: PChar; var Len: LongInt);
    procedure GetName(Buffer: PChar; var Len: LongInt);
    function GetNeeds: TBenchmarkNeeds;
    function Prepare(Host: IBenchmarkHost): TBenchmarkResult;
    function RunBenchmark(ResultMap: IBenchmarkResultMap): TBenchmarkResult;
  end;

  IBenchmarkGuestPas = interface ['{564F19C1-40BC-4D42-AE78-A254AAEABD45}']
    {$CALLING register}
    function GetDescription: String;
    function GetName: String;
    {$I DGLIntfCalling.inc}
  end;

  // The interface to a very accurate timer.
  IBenchmarkTimer = interface ['{B19C407C-4A53-4754-96D5-1F10557BD214}']
    // Reset the timer. This MUST be called before the first call to
    // GetInterval.
    procedure Reset;
    // Returns the time passed since the last call to GetInterval (if any). If
    // GetInterval has not been called yet, it returns the time since the last
    // call to Reset.
    function GetInterval: Double;
  end;

  // Through this interface the benchmark library can expose its benchmarks.
  IBenchmarkHost = interface ['{7CF9A492-3272-42DE-A850-87FF761AF223}']
    procedure Add(Guest: IBenchmarkGuest);
    function GetTimer: IBenchmarkTimer;
  end;

  // This interface is the first a library exposes. It is the gateway which the
  // benchmark uses to communicate with the library.
  // This interface does not resemble a benchmark application. The single apps
  // are identified via a IBenchmarkGuest instance.
  IBenchmarkLib = interface ['{C80D5031-368A-4CFF-8351-9A63054E4E26}']
    procedure Hello(Host: IBenchmarkHost);
  end;

  TBenchmarkLibHello = procedure (out Intf: IBenchmarkLib);

implementation

end.

